import greenfoot.*;
import java.util.List;
import java.awt.Color;
import javax.swing.JOptionPane;

public class LevelEditor extends World
{
    public LevelEditor()
    {    
        super(600, 400, 1, false);
        defaultBackground();
        //addObject(new Base(getWidth()/2-50,"ImpossibleBlock.png"),getWidth()/2,getHeight()-75);
        for (int i=75; i<getWidth()-25; i+=50){
            addObject(new Box(),i,getHeight()-75);
        }
        addObject(new X("LevelEditor"),getWidth()/2,getHeight()/2);
        addObject(new MenuSelection("Press M and N to turn debug mode on and off respectively. Right click to place blocks, and left click to remove them.\nUse the number keys to choose different brushes, and use the fght keys to change the brush type.\nPress the \"i\" key to get information about the currently selected brush.\n-Entity1037","","","",15,Color.BLACK,Color.BLACK),getWidth()/2,30);
        addObject(new DebugInfo(),getWidth()/2,80);
        addObject(new MenuSelection("Main Menu","Are you sure you want to exit the Level Editor?\nAny unsaved data will be lost!","@yn","MainMenu",30,Color.BLACK,Color.YELLOW),getWidth()/2+40,getHeight()-15);
        addObject(new LevelSettingsEditor(),getWidth()-100,getHeight()-15);
        addObject(new Brush(),0,0);
        addObject(new SaveLoadEditorSelection("Save",30,Color.BLACK,Color.YELLOW),50,getHeight()-15);
        addObject(new SaveLoadEditorSelection("Load",30,Color.BLACK,Color.YELLOW),130,getHeight()-15);
        addObject(new SaveLoadEditorSelection("Convert",30,Color.BLACK,Color.YELLOW),220,getHeight()-15);
        addObject(new Fps(),35,50);
    }
    public void defaultBackground(){
        GreenfootImage cell = new GreenfootImage(50, 50);  
        cell.setColor(Color.white);  
        cell.fill();  
        cell.setColor(Color.gray);  
        cell.drawRect(0, 0, 49, 49);
        cell.drawLine(25,0,25,50);
        cell.drawLine(0,25,50,25);
        setBackground(cell);
    }
    public void act(){
        scrollingEngine();
        mouse();
        gridPositionDisplay();
        setPaintOrder(VisualAffects.class,MouseButtons.class,DebugInfo.class,FadeOverlay.class,X.class,Brush.class,BossDoor.class,Enemies.class,xbullets.class,shot3.class,shot2.class,shot1.class,Terrain.class);
    }
    public void gridPositionDisplay(){
        Actor x = (Actor)getObjects(X.class).get(0);
        int xp=x.getX()+xst-llimit;
        int yp=x.getY()+yst-ulimit;
        if (xp>0)xp+=50;
        if (yp>0)yp+=50;
        xp/=50;
        yp/=50;
        ((DebugInfo)getObjects(DebugInfo.class).get(0)).giveGridCoordinates(xp,yp);
    }
    boolean autoScroll=true;
    public void autoScroll(boolean a){autoScroll=a;}
    int xc=getWidth()/2;//X center of screen
    int yc=getHeight()/2;//Y center of screen
    int xX=xc;//X's X coordinate
    int xY=yc;//X's Y coordinate
    public void getCoordinates(int x,int y){xX=x; xY=y;}
    int xcorrect=0;//How much world needs to be scrolled on the X-axis
    int ycorrect=0;//How much world need to be scrolled on the Y-axis
    int s=25;//How far from center X can stray before the screen scrolls
    static int llimit=1000;
    static int rlimit=1000+llimit;
    static int ulimit=500;
    static int dlimit=200+ulimit;
    public static int getLeftLimit(){return llimit;}
    public static int getRightLimit(){return rlimit-llimit;}
    public static int getUpLimit(){return ulimit;}
    public static int getDownLimit(){return dlimit-ulimit;}
    int xst=llimit;//How far the screen's x coordinate is away from start
    int yst=ulimit;//How far the screen's y coordinate is away from start
    public void setScrollingLimits(int left, int right, int up, int down){scrollToBeginning(); xst=left; yst=up; rlimit=right+left; llimit=left; ulimit=up; dlimit=down+up; defaultBackground();}
    public void scrollingEngine(){
        xcorrect=0;
        ycorrect=0;
        if (autoScroll==true){
            //Figure out how far away X is from the scrolling margin, if at all
            if (xX<xc-s){xcorrect=xc-s-xX;}else if (xX>xc+s){xcorrect=xc+s-xX;}
            if (xY<yc-s){ycorrect=yc-s-xY;}else if (xY>yc+s){ycorrect=yc+s-xY;}
        }
        /**if (Greenfoot.isKeyDown("a"))xcorrect=8;
        if (Greenfoot.isKeyDown("d"))xcorrect=-8;
        if (Greenfoot.isKeyDown("w"))ycorrect=8;
        if (Greenfoot.isKeyDown("s"))ycorrect=-8;*/
        //Test if the scrolling boundries have been reached, and stop scrolling if they have
        if (xst+-xcorrect>rlimit)xcorrect=xst-rlimit;
        if (xst+-xcorrect<0)xcorrect=xst;
        if (yst+-ycorrect>dlimit)ycorrect=yst-dlimit;
        if (yst+-ycorrect<0)ycorrect=yst;
        //Track the screen's scrolling
        xst+=-xcorrect; //x-axis tracking
        yst+=-ycorrect; //y-axis tracking
        //Scroll world, if at all
        if (xcorrect==0&&ycorrect==0)return;
        scroll();
    }
    public void scrollToBeginning(){
        autoScroll=false;
        xcorrect=xst-llimit;
        ycorrect=yst-ulimit;
        xst=llimit;
        yst=ulimit;
        scroll();
        List players = getObjects(X.class);
        if (!players.isEmpty()){
            Actor player = (Actor) players.get(0);
            player.setLocation(getWidth()/2,getHeight()/2);
        }
        autoScroll=true;
        defaultBackground();
        repaint();
    }
    Class[] world = {SpriteSheet.class,Terrain.class};
    public void scroll(){
        for (int i=0; i<world.length; i++){
            List objects = getObjects(world[i]);
            if (! objects.isEmpty()){
                for (int o=0; o<objects.size(); o++){
                    Actor object = (Actor) objects.get(o);
                    object.setLocation(object.getX()+xcorrect,object.getY()+ycorrect);
                }
            }
        }
        int gridSize=50;
        int scrollAmtX = (xcorrect+gridSize)%gridSize; // wraps along x-axis  
        int scrollAmtY = (ycorrect+gridSize)%gridSize; // wraps along y-axis  
        GreenfootImage old = new GreenfootImage(gridSize, gridSize);  
        old.drawImage(getBackground(), 0, 0); // the current grid-sized image  
        GreenfootImage image = new GreenfootImage(gridSize, gridSize);  
        image.drawImage(old, scrollAmtX, scrollAmtY);  
        image.drawImage(old, scrollAmtX-gridSize, scrollAmtY);  
        image.drawImage(old, scrollAmtX, scrollAmtY-gridSize);  
        image.drawImage(old, scrollAmtX-gridSize, scrollAmtY-gridSize);  
        setBackground(image);  
    }
    
    int terrain=1;
    int type=0;
    String enemy="";
    public void mouse(){
        MouseInfo mouse = Greenfoot.getMouseInfo();
        if (mouse==null)return;
        
        boolean brushChange=false;
        //Change Brush
        if (Greenfoot.isKeyDown("0")){terrain=0; enemy=JOptionPane.showInputDialog(null,"Note: any enemy name entered that cannot be recognized will\nresult in the enemy turning nto a Metall\n\n-Available Enemies-\nTestOrb\nAwesomeFace");}
        if (Greenfoot.isKeyDown("1"))terrain=1;
        if (Greenfoot.isKeyDown("2"))terrain=2;
        if (Greenfoot.isKeyDown("3"))terrain=3;
        if (Greenfoot.isKeyDown("4"))terrain=4;
        if (Greenfoot.isKeyDown("5"))terrain=5;
        if (Greenfoot.isKeyDown("6"))terrain=6;
        if (Greenfoot.isKeyDown("7"))terrain=7;
        if (Greenfoot.isKeyDown("8"))terrain=8;
        if (Greenfoot.isKeyDown("9"))terrain=9;
        //Change Type
        if (Greenfoot.isKeyDown("t"))type=0;
        if (Greenfoot.isKeyDown("h"))type=1;
        if (Greenfoot.isKeyDown("g"))type=2;
        if (Greenfoot.isKeyDown("f"))type=3;
        if (Greenfoot.isKeyDown("i")){
            if (terrain==0)JOptionPane.showMessageDialog(null,"Enemy Spawn\n\nDesignates where to spawn a certain enemy. One enemy\nwill spawn when the EnemySpawn is scrolled onto the screen.");
            if (terrain==1)JOptionPane.showMessageDialog(null,"BLOCK\n\nBasic terrain object that the player can collide into.");
            if (terrain==2)JOptionPane.showMessageDialog(null,"Base Marker\n\nUsed to mark the dimensions of a Base, which is then\ninserted into the stage after two have been placed.\nIt is basically used to shorten the amount of save data.");
            if (terrain==3)JOptionPane.showMessageDialog(null,"Brick\n\nBasic terrain object that the player can collide into.");
            if (terrain==4)JOptionPane.showMessageDialog(null,"Wall Marker\n\nUsed to mark the dimensions of a Wall, which is then\ninserted into the stage after two have been placed.\nIt is basically used to shorten the amount of saved data.");
            if (terrain==5)JOptionPane.showMessageDialog(null,"Spike\n\nInstantly destroyes a player upon contact. Changing it's type causes it to rotate.");
            if (terrain==6)JOptionPane.showMessageDialog(null,"Spike Sheet Marker\n\nUsed to mark the dimensions of a Spike Sheet, which is\nthen inserted into the stage after two have been placed.\nThis is very useful, due to the fact that you'll probably be\nneeding a lot of spikes placed next to each other.");
            if (terrain==7)JOptionPane.showMessageDialog(null,"Scroll Stopper\n\nPrevents the screen from scrolling so that the screen\nstops as if it collided with it. The scroll stopper should never\nactually come on screen when playing the stage.");
            if (terrain==8)JOptionPane.showMessageDialog(null,"Symbol Block\n\nA block featuring a symbol. This is a throwback to classic\nMegaman where the Wily logo was displayed near the boss door\nat the end of a stage. This block's different types feature\ndifferent symbols.");
            if (terrain==9)JOptionPane.showMessageDialog(null,"Checkpoint Marker\n\nUsed to mark the dimensions of a Checkpoint, which is\nthen inserted into the stage after two have been placed. The player will\nteleport back into the stage in the middle of the checkpoint's image.");
        }
        ((Brush)getObjects(Brush.class).get(0)).changeImage(terrain,type);
        //Snap Terrain to Grid
        int xplace=mouse.getX()+xst;
       int yplace=mouse.getY()+yst;
        if ((terrain>2&&terrain<7)||terrain==9){
            xplace/=25;
            xplace*=25;
            xplace-=xst;
            xplace+=12;
            yplace/=25;
            yplace*=25;
            yplace-=yst;
            yplace+=12;
        }else{
            xplace+=12;
            xplace/=25;
            xplace*=25;
            xplace-=xst;
            yplace+=12;
            yplace/=25;
            yplace*=25;
            yplace-=yst;
        }
        if ((xplace<0&&xst<=0)||(yplace<0&&xst<=0)||(xplace>getWidth()&&xst>=rlimit)||(yplace>getHeight()&&yst>=dlimit))return;
        List objects = getObjectsAt(mouse.getX(),mouse.getY(),Terrain.class);
        List selections = getObjectsAt(mouse.getX(),mouse.getY(),MouseButtons.class);
        if (mouse.getButton()==1&&!objects.isEmpty()){
            removeObject((Actor)objects.get(0));
        }
        if (mouse.getButton()==3&&objects.isEmpty()&&selections.isEmpty()){
            if (terrain==0)addObject(new EnemySpawn(enemy),xplace,yplace);
            if (terrain==1)addObject(new Box(),xplace,yplace);
            if (terrain==2)addObject(new BaseMarker(),xplace,yplace);
            if (terrain==3)addObject(new Brick(),xplace,yplace);
            if (terrain==4)addObject(new WallMarker(),xplace,yplace);
            if (terrain==5)addObject(new Spike(type*90),xplace,yplace);
            if (terrain==6)addObject(new SpikeSheetMarker(type*90),xplace,yplace);
            if (terrain==7)addObject(new ScrollStopper(),xplace,yplace);
            if (terrain==8)addObject(new SymbolBlock(type),xplace,yplace);
            if (terrain==9)addObject(new CheckpointMarker(),xplace,yplace);
        }
    }
}
