import greenfoot.*;
import java.util.List;
import java.awt.Color;
import javax.swing.JOptionPane;
import java.io.*;

public class SaveLoadEditorSelection extends MouseButtons
{
    String action;
    String image;
    String password;
    String world;
    String passwordRequest;
    int size;
    Color normal=Color.BLACK;
    Color highlighted=Color.WHITE;
    int badPassTimer=0;
    public SaveLoadEditorSelection(String sl,int s, Color n, Color h){
        action=sl;
        size=s;
        normal=n;
        highlighted=h;
    }
    public void addedToWorld(World world){
        image=action;
        updateImage(normal);
    }
    UserInfo info;
    public void act()
    {
        MouseInfo mouse = Greenfoot.getMouseInfo();
        Color c;
        if (mouse!=null&&mouse.getX()>getX()-getImage().getWidth()/2&&mouse.getX()<getX()+getImage().getWidth()/2&&mouse.getY()>getY()-getImage().getHeight()/2&&mouse.getY()<getY()+getImage().getHeight()/2){
            c=highlighted;
        }else{
            c=normal;
        }
        if (badPassTimer>0){badPassTimer++;}else{image=action;}
        if (badPassTimer>=200){badPassTimer=0;}
        updateImage(c);
        if (Greenfoot.mouseClicked(this)){
            if (isRunningAsApplet())return;
            String files="||ALL FILES||";;
            File game = new File(System.getProperty("user.dir"));
            File folder = new File(game.getParent()+"/Megaman EX Levels");//Set the directory to where ever you want the Megaman EX Levels folder to be
            if (folder!=null&&!folder.exists()){
                if (JOptionPane.showConfirmDialog(null,"In order to save/load levels, a folder needs to be created.\nWould you like to create one?","WARNING",JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION){
                    folder.mkdir();
                    JOptionPane.showMessageDialog(null,"A new folder has been created to store custom made levels\nat this location: "+folder.getAbsolutePath());
                }
                //System.out.println("Created Folder: "+folder.getAbsolutePath());
            }
            files+=listAllFiles(folder);
            String s="";
            if (action.equals("Load"))s = JOptionPane.showInputDialog(null,"Enter the name of the level you wish to load.\nWARNING: any unsaved data in the current level will\nbe lost!\n\n"+files);
            if (action.equals("Save"))s = JOptionPane.showInputDialog(null,"Enter the name of the level you wish to save to.\nWarning: saving a level with the same name as another will overwrite the other file!\n\n"+files);
            if (!action.equals("Convert")&&(s==null||s.equals("")))return;
            
            if (action.equals("Save")){
                try{
                    File level = new File(folder.getAbsolutePath()+"/"+s+".dat");
                    PrintWriter writer = new PrintWriter(level);
                    ((LevelEditor)getWorld()).scrollToBeginning();
                    writer.println(((LevelEditor)getWorld()).getLeftLimit());
                    writer.println(((LevelEditor)getWorld()).getRightLimit());
                    writer.println(((LevelEditor)getWorld()).getUpLimit());
                    writer.println(((LevelEditor)getWorld()).getDownLimit());
                    ((LevelEditor)getWorld()).scrollToBeginning();
                    List terrain = getWorld().getObjects(Level.class);
                    for (int t=0; t<terrain.size(); t++){
                        Actor object = (Actor) terrain.get(t);
                        writer.println(saveClassData(object));
                    }
                    writer.close();
                    JOptionPane.showMessageDialog(null,"Save Successful!");
                }catch(FileNotFoundException e){JOptionPane.showMessageDialog(null,"Unable to find requested stage."); e.printStackTrace();}
            }else if (action.equals("Load")){
                try{
                    File locatedLevel = new File(folder.getAbsolutePath()+"/"+s+".dat");
                     if (!locatedLevel.exists()){
                        JOptionPane.showMessageDialog(null,"Unable to find requested stage.");
                        return;
                    }
                    deleteCurrentLevel();
                    ((LevelEditor)getWorld()).scrollToBeginning();
                    getWorld().repaint();
                    FileReader readlevel= new FileReader(locatedLevel);
                    BufferedReader level = new BufferedReader(readlevel);
                    String data=level.readLine();
                    int l=Integer.parseInt(data);
                    data=level.readLine();
                    int r=Integer.parseInt(data);
                    data=level.readLine();
                    int u=Integer.parseInt(data);
                    data=level.readLine();
                    int d=Integer.parseInt(data);
                    ((LevelEditor)getWorld()).setScrollingLimits(l,r,u,d);
                    while (true){
                        data=level.readLine();
                        if (data!=null){loadClassData(data);}else{break;}
                    }
                    level.close();
                }catch(Exception e){JOptionPane.showMessageDialog(null,"An error has occured while loading the stage. Check the console window for details."); e.printStackTrace();}
            }else if(action.equals("Convert")){
                System.out.println("\n----Greenfoot Code Level Conversion Output----\n");
                ((LevelEditor)getWorld()).scrollToBeginning();
                System.out.println("\t\tsetScrollingLimits("+((LevelEditor)getWorld()).getLeftLimit()+","+((LevelEditor)getWorld()).getRightLimit()+","+((LevelEditor)getWorld()).getUpLimit()+","+((LevelEditor)getWorld()).getDownLimit()+");");
                List terrain = getWorld().getObjects(Terrain.class);
                for (int t=0; t<terrain.size(); t++){
                    Actor object = (Actor) terrain.get(t);
                    System.out.println(convertClassData(object));
                }
                JOptionPane.showMessageDialog(null,"Conversion Successful!");
            }
        }
    }
    
    private boolean isRunningAsApplet(){
        return null != System.getSecurityManager();
    }
    
    public static String listAllFiles(File folder){
        if (!folder.exists()){
            JOptionPane.showMessageDialog(null,"ERROR, this folder does not exist: "+folder.getAbsolutePath());
            return "";
        }
        String levels="";
        File[] listOfFiles = folder.listFiles();
        for (File f : listOfFiles) {
            if (f.isFile()) {
               levels+="\n"+f.getName().substring(0,f.getName().indexOf("."));
            }
        }
        return levels;
    }
    
    private String saveClassData(Actor actor){
        String data="";
        if (actor instanceof Box){data="b"+(actor.getX()/25)+","+(actor.getY()/25);}
        if (actor instanceof Base){data="a"+(actor.getX()/25)+","+(actor.getY()/25)+","+(actor.getImage().getWidth()/50)+","+(actor.getImage().getHeight()/50);}
        if (actor instanceof Ramp1){data="m"+(actor.getX()/25)+","+(actor.getY()/25);}
        if (actor instanceof Brick){data="r"+actor.getX()+","+actor.getY();}
        if (actor instanceof Wall){data="w"+actor.getX()+","+actor.getY()+","+(actor.getImage().getWidth()/25)+","+(actor.getImage().getHeight()/25);}
        if (actor instanceof Spike){data="s"+actor.getX()+","+actor.getY()+","+(actor.getRotation()/90);}
        if (actor instanceof SpikeSheet){data="p"+actor.getX()+","+actor.getY()+","+(actor.getImage().getWidth()/25)+","+(actor.getImage().getHeight()/25)+","+(actor.getRotation()/90);}
        if (actor instanceof ScrollStopper){data="c"+(actor.getX()/25)+","+(actor.getY()/25);}
        if (actor instanceof Checkpoint){data="t"+actor.getX()+","+actor.getY()+","+(actor.getImage().getWidth()/25)+","+(actor.getImage().getHeight()/25);}
        if (actor instanceof BossDoor){data="d"+actor.getX()+","+actor.getY()+","+((BossDoor)actor).getBoss()+","+((BossDoor)actor).getScrollingStopType()+","; if (((BossDoor)actor).getRoomClearCheck()){data+="1,";}else{data+="0,";} data+=actor.getRotation()/90;}
        if (actor instanceof SymbolBlock){data="y"+(actor.getX()/25)+","+(actor.getY()/25)+","+((SymbolBlock)actor).getType();}
        if (actor instanceof EnemySpawn){data="e"+(actor.getX()/25)+","+(actor.getY()/25)+","+compressEnemyData(((EnemySpawn)actor).getEnemy());}
        System.out.println(data);
        return data+".";
    }
    private String convertClassData(Actor actor){
        String data="\t\t";
        if (actor instanceof Box){data+="addObject(new Box(),"+actor.getX()+","+actor.getY()+");";}
        if (actor instanceof Base){data+="addObject(new Base("+actor.getImage().getWidth()+","+actor.getImage().getHeight()+"),"+actor.getX()+","+actor.getY()+");";}
        if (actor instanceof Ramp1){data+="addObject(new Ramp1(),"+actor.getX()+","+actor.getY()/25+");";}
        if (actor instanceof Brick){data+="addObject(new Brick(),"+actor.getX()+","+actor.getY()+");";}
        if (actor instanceof Wall){data+="addObject(new Wall("+actor.getImage().getWidth()+","+actor.getImage().getHeight()+"),"+actor.getX()+","+actor.getY()+");";}
        if (actor instanceof Spike){data+="addObject(new Spike("+actor.getRotation()+"),"+actor.getX()+","+actor.getY()+");";}
        if (actor instanceof SpikeSheet){data+="addObject(new SpikeSheet("+actor.getImage().getWidth()+","+actor.getImage().getHeight()+","+actor.getRotation()+"),"+actor.getX()+","+actor.getY()+");";}
        if (actor instanceof ScrollStopper){data+="addObject(new ScrollStopper(),"+actor.getX()+","+actor.getY()+");";}
        if (actor instanceof Checkpoint){data+="addObject(new Checkpoint("+actor.getImage().getWidth()+","+actor.getImage().getHeight()+"),"+actor.getX()+","+actor.getY()+");";}
        if (actor instanceof BossDoor){data+="addObject(new BossDoor(\""+((BossDoor)actor).getBoss()+"\","+((BossDoor)actor).getScrollingStopType()+","+((BossDoor)actor).getRoomClearCheck()+","+actor.getRotation()+"),"+actor.getX()+","+actor.getY()+");";}
        if (actor instanceof SymbolBlock){data+="addObject(new SymbolBlock("+((SymbolBlock)actor).getType()+"),"+actor.getX()+","+actor.getY()+");";}
        if (actor instanceof EnemySpawn){data+="addObject(new EnemySpawn(\""+compressEnemyData(((EnemySpawn)actor).getEnemy())+"\"),"+actor.getX()+","+actor.getY()+");";}
        return data;
    }
    
    private String compressEnemyData(String e){
        if (e.equalsIgnoreCase("TestOrb"))return "aw";
        if (e.equalsIgnoreCase("AwesomeFace"))return "af";
        return "me";
    }
    private String decompressEnemyData(String e){
        if (e.equalsIgnoreCase("aw"))return "TestOrb";
        if (e.equalsIgnoreCase("af"))return "AwesomeFace";
        return "Metall";
    }
    private String capsFix(String e){
        if (e.equalsIgnoreCase("TestOrb"))return "TestOrb";
        if (e.equalsIgnoreCase("AwesomeFace"))return "AwesomeFace";
        if (e.equalsIgnoreCase("Metall"))return "Metall";
        return null;
    }
    
    /**Error occures were the object variable refills and the 
       data variable is empty. Then, an StringIndexOutOfBoundsException
       occurs on line 138*/
    private void loadClassData(String objects){
        if (objects.indexOf(".")==-1)return;
        for (int t=0; t<20; t++){
            String data=objects.substring(0,objects.indexOf("."));
            //JOptionPane.showMessageDialog(null,"objects="+objects+"\ndata="+data);
            int periods=0;
            for (int i=0; i<objects.length(); i++){
                if (objects.substring(i,i+1).equals("."))periods++;
            }
            if (periods>1){objects=objects.substring(objects.indexOf(".")+1);}
            if (data.length()==0)return;
            String o="";
            o=data.substring(0,1);
            data=data.substring(1);
            int x=0, y=0, a=0, b=0, c=0;
            //Get data of objects and add it to the world
            if (o.equals("b")){
                x=getInt(data)*25;
                data=clip(data);
                y=getInt(data)*25;
                getWorld().addObject(new Box(),x,y);
            }
            if (o.equals("a")){
                x=getInt(data)*25;
                data=clip(data);
                y=getInt(data)*25;
                data=clip(data);
                a=getInt(data)*50;
                data=clip(data);
                b=getInt(data)*50;
                getWorld().addObject(new Base(a,b),x,y);
            }
            if (o.equals("r")){
                x=getInt(data);
                data=clip(data);
                y=getInt(data);
                data=clip(data);
                getWorld().addObject(new Brick(),x,y);
            }
            if (o.equals("w")){
                x=getInt(data);
                data=clip(data);
                y=getInt(data);
                data=clip(data);
                a=getInt(data)*25;
                data=clip(data);
                b=getInt(data)*25;
                getWorld().addObject(new Wall(a,b),x,y);
            }
            if (o.equals("s")){
                //LEAVE COORDINATES AS IS!!!
                x=getInt(data);
                data=clip(data);
                y=getInt(data);
                data=clip(data);
                a=getInt(data);
                getWorld().addObject(new Spike(a*90),x,y);
            }
            if (o.equals("p")){
                //LEAVE COORDINATES AS IS!!!
                x=getInt(data);
                data=clip(data);
                y=getInt(data);
                data=clip(data);
                a=getInt(data)*25;
                data=clip(data);
                b=getInt(data)*25;
                data=clip(data);
                c=getInt(data);
                getWorld().addObject(new SpikeSheet(a,b,c*90),x,y);
            }
            if (o.equals("c")){
                x=getInt(data)*25;
                data=clip(data);
                y=getInt(data)*25;
                getWorld().addObject(new ScrollStopper(),x,y);
            }
            if (o.equals("d")){
                x=getInt(data);
                data=clip(data);
                y=getInt(data);
                data=clip(data);
                String name=data.substring(0,data.indexOf(","));
                data=clip(data);
                a=getInt(data);
                data=clip(data);
                b=getInt(data);
                data=clip(data);
                c=getInt(data)*90;
                getWorld().addObject(new BossDoor(name,a,(b==1),c),x,y);
            }
            if (o.equals("y")){
                //KEEP X AND Y COORDINATES AS IS!!!
                x=getInt(data);
                data=clip(data);
                y=getInt(data);
                data=clip(data);
                a=getInt(data);
                getWorld().addObject(new SymbolBlock(a),x*25,y*25);
            }
            if (o.equals("e")){
                x=getInt(data)*25;
                data=clip(data);
                y=getInt(data)*25;
                data=clip(data);
                data=decompressEnemyData(data);
                getWorld().addObject(new EnemySpawn(data),x,y);
            }
            if (o.equals("t")){
                x=getInt(data);
                data=clip(data);
                y=getInt(data);
                data=clip(data);
                a=getInt(data)*25;
                data=clip(data);
                b=getInt(data)*25;
                getWorld().addObject(new Checkpoint(a,b),x,y);
            }
            //JOptionPane.showMessageDialog(null,x+"\n"+y+"\n"+a+"\n"+b);
            getWorld().repaint();
            if (periods<2)break;
        }
    }
    public int getInt(String data){
        int integer=0;
        if (data.indexOf(",")!=-1){
            integer=Integer.parseInt(data.substring(0,data.indexOf(",")));
        }else{
            integer=Integer.parseInt(data);
        }
        return integer;
    }
    public String clip(String data){
        return data.substring(data.indexOf(",")+1);
    }
    
    public void deleteCurrentLevel(){
        List terrain = getWorld().getObjects(Terrain.class);
        for (int t=0; t<terrain.size(); t++){
            Actor object = (Actor) terrain.get(t);
            getWorld().removeObject(object);
        }
        getWorld().repaint();
    }
    
    public void updateImage(Color c){
        setImage(new GreenfootImage(image,size,c,new Color(0,0,0,0)));
    }
}